// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2022 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.


#include "LesCourbes1D.h"
#include "ParaGlob.h"
//#include <extras.h>
#include "CharUtil.h"

// --------------- variables statiques ---------
 MotCle LesCourbes1D::motCle; // liste des mots clés

LesCourbes1D::LesCourbes1D () :  // par defaut
    listeDeCourbe1D()
  #ifdef UTILISATION_MPI
    // pour les i/o
    ,buffer_ioBI_MPI(),tailles_buffer_ioBI_MPI(),offset_LesCourbes1D()
    ,nb_buffer(1),taille_offset(2)
  #endif
    {
  #ifdef UTILISATION_MPI
      buffer_ioBI_MPI.Change_taille(nb_buffer);
      tailles_buffer_ioBI_MPI.resize(nb_buffer,0);
      offset_LesCourbes1D.resize(taille_offset);
  #endif
    };
// DESTRUCTEUR :
LesCourbes1D::~LesCourbes1D ()
    { // effacement des grandeurs pointées par la liste de référence
      map < string, Courbe1D * , std::less <string> >::iterator i,ifin=listeDeCourbe1D.end();
      for (i=listeDeCourbe1D.begin() ; i != ifin; i++)
             if ((*i).second != NULL) delete (*i).second;
     };
        
 // lecture des courbes
 void LesCourbes1D::Lecture(UtilLecture & entreePrinc)
   { if (strstr(entreePrinc.tablcar,"les_courbes_1D")!=NULL)
      {try
        { if (ParaGlob::NiveauImpression() >= 4) cout << " debut de la lecture de courbes " << endl;
          // lecture des courbes temps qu'il n'y a pas de nouveau mot clé
          string nom_courbe,type_courbe; // variables de travail
          Courbe1D * courbe=NULL;
          entreePrinc.NouvelleDonnee();
          while (!motCle.SimotCle(entreePrinc.tablcar))
            { // lecture du nom de la courbe et de son type
              *(entreePrinc.entree) >> nom_courbe >> type_courbe;
              if (ParaGlob::NiveauImpression() >= 6) 
                 cout << " lecture de la courbe " << nom_courbe << endl;
              // création de la courbe    
              courbe = Courbe1D::New_Courbe1D(nom_courbe,Id_Nom_Courbe1D (type_courbe.c_str()));
              // lecture de la courbe
              courbe->LectDonnParticulieres_courbes (nom_courbe,&entreePrinc);
              if (ParaGlob::NiveauImpression() >= 6) 
                 cout << " fin de la lecture de la courbe " << nom_courbe << endl;
              // verification que l'on n'a pas deux noms de courbes identiques
              bool res = false;
              map < string, Courbe1D * , std::less <string> > ::iterator il = listeDeCourbe1D.find(nom_courbe);
              if (il != listeDeCourbe1D.end()) res=true;                                          
              if (res)
                { cout <<" \n Erreur de lecture de courbes 1D, deux courbes ont le meme nom \n";
                  cout <<" nom = " << nom_courbe << '\n';
                  cout <<"LesCourbes1D::LectureLesCourbes1D(UtilLecture & entreePrinc)" 
                       << endl;
                  entreePrinc.MessageBuffer("lecture des  courbes1D");                                    
                  throw (UtilLecture::ErrNouvelleDonnee(-1));
                  Sortie (1);
                 }
              else
                listeDeCourbe1D[nom_courbe]=courbe;
              entreePrinc.NouvelleDonnee(); // nouvelle ligne                    
              };
           if (ParaGlob::NiveauImpression() >= 4) cout << " fin de la lecture de courbes " << endl;
           }
       catch (ErrSortieFinale)
            // cas d'une direction voulue vers la sortie
            // on relance l'interuption pour le niveau supérieur
          { ErrSortieFinale toto;
            throw (toto);
          }
       catch(...)
        { cout << "\n ** erreur en lecture d'une courbes 1D ";
          throw (UtilLecture::ErrNouvelleDonnee(-1));
          Sortie(1);
        };
      };
     // maintenant on établit les liens éventuels entres les courbes qui peuvent dépendre les unes des autres
     // (exemple de courbe complexe construite à partir de courbe simples) 
     map < string, Courbe1D * , std::less <string> >::iterator idl,edlfin=listeDeCourbe1D.end();
     for (idl = listeDeCourbe1D.begin();idl != edlfin;idl++)
      {Courbe1D * co = (*idl).second; // pour plus de clarté 
       if (co->DependAutreCourbes())
         {// cas d'une courbe avec dépendance
          // -- récupération des noms de courbes de dépendance
          list <string> lico;
          co->ListDependanceCourbes(lico); 
          // -- création de la liste de pointeur de courbes correspondantes
          list <string>::iterator ili,ilifin=lico.end();
          list <Courbe1D *> liptco;
          for (ili=lico.begin();ili!=ilifin;ili++)
            liptco.push_back(LesCourbes1D::Trouve(*ili));
          // ---- on renseigne la courbe
          co->Lien_entre_courbe(liptco);
          };
       };
     };
    
// affichage et definition interactive des commandes 
void LesCourbes1D::Info_commande_lesCourbes1D(UtilLecture & entreePrinc)
 { string rep;
   cout << "\n voulez-vous utiliser des courbes 1D ? (rep o ou n (defaut)) ";
   rep = lect_return_defaut(false,"n");
   if (rep == "o")
    { // on récupère la liste des identificateurs de courbes actuellement disponible
      list <EnumCourbe1D> list_enuCourbe1D = Courbe1D::Liste_courbe_disponible();
      // affichage et choix de courbe1D
      cout << "\n Donner le type de courbe que vous souhaitez utiliser: ?  "
           << "\n (NB: l'utilitaire va definir un exemple de courbe qu'il faudra ensuite adapter !)   ";;
      string rep;int num; bool choix_valide = false;
      Tableau <bool> chooi(list_enuCourbe1D.size(),false); // tableau de travail
      list <string> nom_de_fichiers; bool inclusion_fichier=false;
      list <EnumCourbe1D>::iterator it,itfin=list_enuCourbe1D.end();
      while (!choix_valide)
      {
       try 
        { // affichage des éléments possibles
          int undeux=0;
          cout << "\n (0 ou f) fin \n";
          int inu;
          for (it=list_enuCourbe1D.begin(),inu=1;it!=itfin;it++,inu++)
            { cout << "\n";
              cout << " (" << inu << ") " << Nom_Courbe1D(*it) << "   ";
              //undeux++;
              //if (undeux==2) {cout << "\n";undeux=0;}
             };
          int nb_courbe = list_enuCourbe1D.size();
          cout << "\n (" << nb_courbe+1 << ")   inclusion d'un fichier de def de courbe ";
          cout << "\n un numero ? ";
          rep = lect_return_defaut(false,"f");
          if (rep == "fin_prog") Sortie(1);
          num = ChangeEntier(rep);
          if (Minuscules(rep) == "f")
            num = 0;
          choix_valide=false;  
          if (num == 0) 
           { choix_valide=true;}
          else if (num == nb_courbe+1)
           { string nom_fichier="";
             cout << "\n nom du fichier ? "; nom_fichier = lect_chaine();
             cout << " nom lu: "<<nom_fichier << "\n";
             nom_de_fichiers.push_back(nom_fichier);
             inclusion_fichier=true;
           }
          else // sinon
           { if ((num >= 0)&&(num<=list_enuCourbe1D.size())) 
              {if (chooi(num))
                 cout << "\n type d'element deja choisit, recommencer";
               else    
                chooi(num) = true;
               }
             else { cout << "\n Erreur on attendait un entier entre 0 et "<< list_enuCourbe1D.size()+1 <<" !!, "
                         << "\n redonnez une bonne valeur"
                         << "\n ou taper fin_prog pour arreter le programme"; 
                  };
            };
         }    
       catch (ErrSortieFinale)
            // cas d'une direction voulue vers la sortie
            // on relance l'interuption pour le niveau supérieur
          { ErrSortieFinale toto;
            throw (toto);
          }
       catch (...)//(UtilLecture::ErrNouvelleDonnee erreur)
          {  cout << "\n Erreur on attendait un entier entre 0 et "<< list_enuCourbe1D.size() <<" !!, "
                  << "\n redonnez une bonne valeur"
                  << "\n ou taper fin_prog pour arreter le programme";
             choix_valide=false;
          };
       } //-- fin du while 
     // maintenant on définit un exemplaire de chaque courbe choisit
     int cot=1;string nom_courbe; Courbe1D * courbe=NULL;
     for (it=list_enuCourbe1D.begin();it!=itfin;it++,cot++)
      { if(chooi(cot))
         { nom_courbe = "exemple_courbe";
           nom_courbe += ChangeEntierSTring(cot);
           courbe = Courbe1D::New_Courbe1D(nom_courbe,*it);
           listeDeCourbe1D[nom_courbe]=courbe;
          }
       }
     // affichage des infos dans le fichier de commande
     ofstream & sort = *(entreePrinc.Commande_pointInfo()); // pour simplifier
     sort << "\n#-------------------------------------"
          << "\n# definition facultative de courbe 1D|" 
          << "\n#-------------------------------------"
          << "\n \n            les_courbes_1D #------------"
          << endl;
     // puis chaque courbe sort ses infos
     map < string, Courbe1D * , std::less <string> >::iterator itc,itcfin =  listeDeCourbe1D.end();     
     for (itc= listeDeCourbe1D.begin();itc!=itcfin;itc++)
       (*itc).second->Info_commande_Courbes1D(entreePrinc);
     // dans le cas où il y a des inclusions de fichiers
     if (inclusion_fichier)     
       { list <string>::iterator ill,illfin = nom_de_fichiers.end();
         for (ill=nom_de_fichiers.begin();ill != illfin;ill++)
           sort << "\n < " << (*ill) ;
       };

     // fin def courbe
     sort << "\n#------------- fin def des courbes 1D ------------------------"
          << endl;     
     sort << flush;
    };
 };
          
     
 // affichage des courbes
void LesCourbes1D::Affiche() const 
  {   map < string, Courbe1D * , std::less <string> >::const_iterator i,ifin=listeDeCourbe1D.end();
      bool res = false;
      cout << " \n ------- affichage des listes de courbes ------- \n "; 
      for (i=listeDeCourbe1D.begin() ; i != ifin; i++)
          { cout << "\n";
            ((*i).second)->Affiche();
           } 
      cout << endl;          
  };
  
// test si la courbe de nom st1 existe reellement
// retourne false si n'existe pas , true sinon
bool LesCourbes1D::Existe(const string & st1) const 
  { // verification 
    bool res = false;
    map < string, Courbe1D *, std::less <string> > ::const_iterator il = listeDeCourbe1D.find(st1);
    if (il != listeDeCourbe1D.end()) res=true;
    return res;      
   };       

 // retourne la courbe correspondant a une cle
Courbe1D* LesCourbes1D::Trouve(const string & st1) const 
  { map < string, Courbe1D *, std::less <string> > ::const_iterator il = listeDeCourbe1D.find(st1);
    if (il != listeDeCourbe1D.end()) 
        return ((*il).second);
    cout << " \n pb la courbe1D de nom " << st1 << " n\'existe pas !! " << endl;
    Sortie (1);
    Courbe1D* bidon=NULL ;   // pour ne pas avoir de message de warning
    return  bidon;
   };       

// vérification que tout est ok, pres à l'emploi
// ramène true si ok, false sinon
bool LesCourbes1D::Complet()
  { // on passe en revue toutes les courbes
    map < string, Courbe1D *, std::less <string> > ::const_iterator il, ilfin = listeDeCourbe1D.end();
    bool retour = true;
    for (il=listeDeCourbe1D.begin();il!=ilfin;il++)
     { bool ret=((*il).second)->Complet_courbe();
       if (!ret)
        { cout << "\n erreur!!! la courbe n'est pas complete"; 
          ((*il).second)->Affiche();
          retour = false;
         }
      } 
    return retour;  
   };
   
    // utilitaire pour lire une courbe, soit qui ne sera pas stocké par LesCourbes1D
    // dans ce cas sont nom est "_", soit son nom correspond à une ref de nom de courbes existante
    // dans tous les cas on lit un nom et un type de courbe c-a-d un string et un EnumCourbe1D
    //                 -----     différents cas  -------
    // ptcourbe: soit == NULL, : on lit un nom, et on lit un type de courbe
    //                           1) si celui-ci est "_", cela signifie que la courbe à lire
    //                           est interne à l'utilisateur, on crée une courbe, on lit la courbe
    //                           avec Lecture_base_info de la courbe et on ramène un pointeur sur la 
    //                           nouvelle courbe 
    //                           2) si celui-ci est différent de "_", c'est une référence de courbe
    //                           on lit la référence et on ramène un pointeur sur la courbe de 
    //                           LesCourbes1D correspondant 
    //                           
    // soit == une courbe existante: on lit un nom, et on regarde le nom actuel de la courbe pointée
    //                               par ptcourbe que l'on appelera nom_ref
    //    1) nom = "_" et nom_ref = "_" 
    //                 1-a les deux courbes sont du même type on relie les données avec Lecture_base_info
    //                     et on ramène ptcourbe
    //                 1-b les deux courbes sont de type différent, on supprime la courbe pointée par
    //                     ptcourbe, on en crée une nouvelle adoc, et on ramène un pointeur dessus
    //    2) nom != "_" et nom_ref == "_" 
    //                  la courbes pointé par ptcourbe est supprimé, et on associe le pointeur de retour
    //                  a la courbe correspondant à nom de LesCourbes1D 
    //    3) nom == "_" et nom_ref != "_" 
    //                  on crée une nouvelle courbe adoc, on lie avec Lecture_base_info, et on ramène
    //                  un pointeur sur la courbe ainsi crée
    
Courbe1D * LesCourbes1D::Lecture_pour_base_info(istream& ent,const int cas,Courbe1D * ptcourbe)
  {  EnumCourbe1D type_de_courbe;string nom_courbe;
     ent >> nom_courbe >> type_de_courbe; 
     if (ptcourbe == NULL)
      {if (nom_courbe == "_")
        {// cas d'une courbe interne à l'appelant et pas de courbe défini, on en définit une
         ptcourbe = Courbe1D::New_Courbe1D(nom_courbe,type_de_courbe);
         // lecture des infos       
         ptcourbe->Lecture_base_info(ent,cas);
         }
       else
        {// cas d'une référence de courbe
         if (this->Existe(nom_courbe)) 
           { ptcourbe = this->Trouve(nom_courbe);}
         else
          {cout << "\n erreur 1 en lecture de la courbe  de nom " << nom_courbe
                << " elle n'existe pas dans la liste des courbes avec reference"
                << "\n LesCourbes1D::Lecture_pour_base_info(...";
           Sortie(1);     
           } 
          };
       }      
     else 
      { // cas où une courbe est déjà pointé
        if ((ptcourbe->NomCourbe() == "_") && (nom_courbe == "_"))
         { if (type_de_courbe == ptcourbe->Type_courbe())
            {// même type on se contente de relire
             ptcourbe->Lecture_base_info(ent,cas);
             }
           else
            {// pas de même type on supprime, on recrée, on li
             delete ptcourbe;     
             ptcourbe = Courbe1D::New_Courbe1D(nom_courbe,type_de_courbe);
             ptcourbe->Lecture_base_info(ent,cas);
             }
          }
        else if ((ptcourbe->NomCourbe() == "_") && (nom_courbe != "_"))
         { // l'ancien est interne et le nouveau est une ref, on supprime, on récup la ref
           delete ptcourbe;     
           if (this->Existe(nom_courbe)) 
            { ptcourbe = this->Trouve(nom_courbe);}
           else
            {cout << "\n erreur 2 en lecture de la courbe  de nom " << nom_courbe
                  << " elle n'existe pas dans la liste des courbes avec reference"
                  << "\n LesCourbes1D::Lecture_pour_base_info(...";
             Sortie(1);     
             } 
          }  
        else if ((ptcourbe->NomCourbe() != "_") && (nom_courbe == "_"))
         { // l'ancien est une ref et le nouveau est interne, on crée et on li
           ptcourbe = Courbe1D::New_Courbe1D(nom_courbe,type_de_courbe);
           ptcourbe->Lecture_base_info(ent,cas);
          }
       } //-- fin du cas ou ptcourbe != NULL 
   // retour du pointeur
   return ptcourbe;            
 };
  
    // écriture pour base info
    // c'est le pendant de  Lecture_pour_base_info, de manière à être cohérent
void LesCourbes1D::Ecriture_pour_base_info(ostream& sort,const int cas,Courbe1D * ptcourbe)
 { // la courbe d'écrouissage
   sort << ptcourbe->NomCourbe() << " " << ptcourbe->Type_courbe() << " ";
   // écriture le la courbe elle-même que si le nom est "_"
   if (ptcourbe->NomCourbe() == "_")
     {ptcourbe->Ecriture_base_info(sort,cas);};
  };
	 
     
//----- lecture écriture de base info -----
//   lecture base info
// = 1 : on récupère tout
// = 2 : on récupère uniquement les données variables (supposées comme telles)
void LesCourbes1D::Lecture_base_info(istream& entr,const int cas)
 {bool mono_proc_on_lit_un_BI_multiProc = false;

#ifdef UTILISATION_MPI
  int proc_en_cours = ParaGlob::Monde()->rank();
  // === première partie commune à tous les proc
  // normalement tous les buffers ont été rempli, on se contente de rediriger les flux
   
  if (tailles_buffer_ioBI_MPI[0] > 0)
   {// on crée le flux sur le premier buffer:
    istrstream * entrons = new istrstream(buffer_ioBI_MPI(1).c_str(),tailles_buffer_ioBI_MPI[0]) ;
    istream& entr = *entrons; // on redéfinit ent
#endif
    string toto;int taille;
    
    entr >> toto ; // entête
    // dans le cas où on lit un fichier créé avec MPI io, et que l'on n'est pas
    // en MPI, on passe les offsets et tailles
  #ifndef UTILISATION_MPI
    if (toto == "buf0_LesCourbes1D")
     {for (int i=0;i<3;i++) //
       {toto.clear();
        std::getline(entr, toto);
       };
      mono_proc_on_lit_un_BI_multiProc = true;
     };
  #endif
    if (mono_proc_on_lit_un_BI_multiProc) // cas de la lecture en mono
     {entr >> toto >> taille;}   // d'un fichier créé en multi proc
    else // cas d'une lecture en mono d'un fichier mono
         // ou cas d'une lecture multi d'un fichier créé en multi proc
     {entr >> taille;}; //  on a déjà lue toto

    if (cas == 1)
     {if (ParaGlob::NiveauImpression() >= 6)
        cout << " == lecture des courbes \n";
      // on boucle sur le nombre de courbe à lire
      EnumCourbe1D type_de_courbe; //   "
      Courbe1D* courbe; string nom_courbe; string type_courbe; //               "
      
      for (int i=1;i<= taille; i++)
       { // lecture du nom de la courbe et de son type
         entr >> nom_courbe >> type_de_courbe;
         // création de la courbe
         courbe = Courbe1D::New_Courbe1D(nom_courbe,type_de_courbe);
         // lecture de la courbe
         courbe->Lecture_base_info(entr,cas);
         // verification que l'on n'a pas deux noms de courbes identiques
         bool res = false;
         map < string, Courbe1D* , std::less <string> > ::iterator il = listeDeCourbe1D.find(nom_courbe);
         if (il != listeDeCourbe1D.end()) res=true;

         if (res)
          { cout <<" \n Erreur de lecture de courbes 1D, deux courbes ont le meme nom \n";
            cout <<" nom = " << nom_courbe << '\n';
            cout <<"LesCourbes1D::Lecture_base_info(..." << endl;
            Sortie (1);
           }
         else
           // intégration dans la liste
           listeDeCourbe1D[nom_courbe]=courbe;
         }
       }
#ifdef UTILISATION_MPI
   }; // fin lecture  buffer
#endif
 };
        
//   écriture base info   
// = 1 : on sauvegarde tout
// = 2 : on sauvegarde uniquement les données variables (supposées comme telles)
void LesCourbes1D::Ecriture_base_info(ostream& sort,const int cas)
 {
#ifdef UTILISATION_MPI
  {int proc_en_cours = ParaGlob::Monde()->rank();
   // 1) dans le cas d'un proc 0 ou i, on redéfinit le flux de sortie, pour bufferiser les infos en sortie
   // qui sont ensuite sortie via MPI i-o à l'aide de la méthode Ecriture_base_info_MPI_IO
   // les récupérations et les sorties sont effectuées de manière asynchrone par rapport aux autres proc
   // 2) seul le proc 0 sort pour le cas 1, rien pour cas 2
   std::ostringstream sort; // redéfinition de sort
   if (proc_en_cours == 0) // partie dédiée proc 0
    {
#endif
     sort << "\n \n****les_courbes1D_:_nombre= " << listeDeCourbe1D.size() ;
     if (cas == 1)
      {// on balaie la liste
       map < string, Courbe1D* , std::less <string> >::iterator i,ifin=listeDeCourbe1D.end();
       for (i=listeDeCourbe1D.begin() ; i != ifin; i++)
        { // on écrit le nom de la courbe et de sont type, sous forme énuméré, car ensuite dans chaque courbe le nom
          // en string est écrit
          sort << "\n" << (*i).first << " " << ((*i).second)->Type_courbe();
          // puis écriture de la courbe
          ((*i).second)->Ecriture_base_info(sort,cas);
         }
        }
       sort << "\n";
#ifdef UTILISATION_MPI
    };
   buffer_ioBI_MPI(1) = sort.str(); // on sauvegarde
   tailles_buffer_ioBI_MPI[0] = buffer_ioBI_MPI(1).size();
  };
#endif
  };

#ifdef UTILISATION_MPI
// retour de la taille globale des sorties retardées
// concernant les sorties sur .BI
MPI_Offset LesCourbes1D::Taille_buffer_sortie_BI()
  {MPI_Offset taille_totale = 0; // init
   int proc_en_cours = ParaGlob::Monde()->rank();
   if (proc_en_cours == 0) // partie dédiée proc 0
     {// tout d'abord on prend en compte les buffers propres
      for (MPI_Offset i=1;i<= nb_buffer;i++) // normalement 1
        taille_totale += buffer_ioBI_MPI(i).size();
      // puis on intègre la taille de l'ensemble des offsets:
      MPI_Offset taille_entete = 42 + (taille_offset)*21+27; // cf. LesCourbes1D::Ecriture_base_info_MPI_IO
      taille_totale += taille_entete;
     };
   // retour
   return taille_totale; // seule la valeur retournée par proc 0 est licite
  };

// récupération de l'offset global sur .BI pour la classe
// et mise à jour des offsets de classes dérivées si besoin
void LesCourbes1D::Mise_a_jour_Offset_BI_ecriture(MPI_Offset offset,const int cas)
 {// seul le proc 0 calcule les offsets qui sont ensuites transmis à tous les proc
  // mais il faut que tous les proc passent par toutes les méthodes
  int proc_en_cours = ParaGlob::Monde()->rank();
  if (proc_en_cours == 0)
   {
    // on commence par prendre en compte l'offset général puis ceux propres à proc 0
    // ce sont des données qui se situent avant ceux des proc i
    offset_LesCourbes1D[0] = offset;
    // puis la première partie propre à proc 0
    MPI_Offset a_ajouter = offset+tailles_buffer_ioBI_MPI[0];
    // puis on intègre la taille de l'ensemble des offsets:
    MPI_Offset taille_entete = 42 + (taille_offset)*21+27; // cf. LesCourbes1D::Ecriture_base_info_MPI_IO
    offset_LesCourbes1D[1] = taille_entete+a_ajouter; // position final +1
   };
     
  // on transmets à tous les proc i les offsets
  broadcast(*ParaGlob::Monde(), offset_LesCourbes1D, 0);
 };
// idem en lecture, mais là il s'agit uniquement de la mise à jour
// de l'offset seul puis c'est la classe qui s'en débrouille
void LesCourbes1D::Mise_a_jour_Offset_BI_lecture(MPI_Offset offset,const int cas)
  { offset_LesCourbes1D[0] = offset;
  };

// lecture MPI des infos bufferisées
void LesCourbes1D::Lecture_base_info_MPI_IO(MPI_File * ent_MPI,const int cas)
 { int proc_en_cours = ParaGlob::Monde()->rank();

  // tous les proc lisent
  // 1) ==== la taille de l'ensemble des offsets
  MPI_Offset taille_buf = 0; // init
  // on encapsule pour ré-utiliser la même trame
  { char * buffer_car = new char [42] ; // def du tableau de travail
    MPI_Status status;
    int ierr = MPI_File_read_all(*ent_MPI, buffer_car, 42, MPI_CHAR, &status);
    // on crée un flux sur le tableau de caractères:
    istrstream * ent_inter = new istrstream(buffer_car,42) ;
    // lecture de l'entête
    string toto;
    (*ent_inter) >> toto;
    if (toto != "buf0_LesCourbes1D" )
      {cout << "\n **** erreur on devait lire : 'buf0_LesCourbes1D' "
            << " et on lue "<< toto << " la suite n'est pas possible -> arret !! "
            << "\n LesCourbes1D::Lecture_base_info_MPI_IO(... "<<endl;
       Sortie(1);
      };
    // on lit la taille de buf2
    (*ent_inter)  >> taille_buf;
  };
  // 2) ==== lecture de la taille de l'ensemble des offsets
  { char * buffer_car = new char [taille_buf] ; // def du tableau de travail
    MPI_Status status;
    int ierr = MPI_File_read_all(*ent_MPI, buffer_car, taille_buf, MPI_CHAR, &status);
    if (ierr)
      {cout << "\n *** erreur d'acces a MPI_File_read_all: code " << ierr << " , stop Herezh "
            << "\n proc "<<proc_en_cours << ", lecture  de la taille de l'ensemble des offsets "
            << "\n LesCourbes1D::Lecture_base_info_MPI_IO(..." << endl;
       Sortie(1);
      };
    // on crée un flux sur le tableau de caractères:
    istrstream * ent_inter = new istrstream(buffer_car,taille_buf) ;
    // lecture de l'entête
    string toto;
    (*ent_inter) >> toto;
    if (toto != "offset_base_info_MPI_IO" )
      {cout << "\n **** erreur on devait lire : 'offset_base_info_MPI_IO' "
            << " et on lue "<< toto << " la suite n'est pas possible -> arret !! "
            << "\n LesCourbes1D::Lecture_base_info_MPI_IO(... "<<endl;
       Sortie(1);
      };
    // on lit les offsets
    MPI_Offset taille_offset = 2;
    for (MPI_Offset i=0;i<taille_offset;i++)
      (*ent_inter) >>  offset_LesCourbes1D[i];

   };

  // 3) ==== on dimensionne  les_taille_buffer_ioBI_MPI
  { MPI_Offset taille_entete = 42 + (taille_offset)*21+27; // cf. Ecriture_base_info_MPI_IO

    tailles_buffer_ioBI_MPI[0] = offset_LesCourbes1D[1] - (offset_LesCourbes1D[0]+taille_entete);
   };
  
  // 3) ==== lecture du  buffer
  if (tailles_buffer_ioBI_MPI[0] > 0)
  { char * buffer_car = new char [tailles_buffer_ioBI_MPI[0]] ; // def du tableau de travail
    MPI_Status status;
    int ierr = MPI_File_read_all(*ent_MPI, buffer_car, tailles_buffer_ioBI_MPI[0], MPI_CHAR, &status);
    if (ierr)
      {cout << "\n *** erreur d'acces a MPI_File_read_all: code " << ierr << " , stop Herezh "
            << "\n proc "<<proc_en_cours << ", lecture  buffer1 "
            << "\n LesCourbes1D::Lecture_base_info_MPI_IO(..." << endl;
       Sortie(1);
      };
    buffer_ioBI_MPI(1).resize(tailles_buffer_ioBI_MPI[0]);
    buffer_ioBI_MPI(1).replace(0,tailles_buffer_ioBI_MPI[0],buffer_car);
  };
 };

  
// écriture MPI des infos bufferisées
void LesCourbes1D::Ecriture_base_info_MPI_IO(MPI_File * sort_MPI, const int cas)
  { int proc_en_cours = ParaGlob::Monde()->rank();
    // si proc 0, on sort les ref
    if (proc_en_cours == 0)
     {//  === écriture dans flot string de la taille des buffers propres
      // pour avoir une sortie lisible, on crée tout d'abord un buffer string avec les infos
      std::ostringstream buf2(std::ios_base::out);
      buf2 << "\n offset_base_info_MPI_IO \n"; // 27 char
      for (MPI_Offset i=0;i<taille_offset;i++)
        // on sort donc (1)*21 char
        buf2 << std::right << setw(20) << offset_LesCourbes1D[i] << " ";
       
      // le buffer du stream n'est pas pérenne il faut le récupérer
      string buf_inter = buf2.str(); // on récupère le buffer
      MPI_Offset taille_buf = buf_inter.size();
      #ifdef MISE_AU_POINT
       if (taille_buf != (taille_offset)*21+27)
         {cout << "\n *** sans doute erreur, taille_buf "<<taille_buf
               << " est  different de (taille_offset)*21+27 = " << (taille_offset)*21+27
               << "\n  LesCourbes1D::Ecriture_base_info_MPI_IO(..." << endl;
          Sortie(1);
         };
      #endif

      // 1) ==== écriture de la taille de l'ensemble des offsets propres
      // on encapsule pour ré-utiliser la même trame
      {
        std::ostringstream bufdeb(std::ios_base::out);
        bufdeb << "\n buf0_LesCourbes1D  "; // 21 char
        bufdeb << std::right << setw(20) << taille_buf; // on justifie à droite et on utilise 20 char
        bufdeb << " "; // 1 char
        string buf_inter = bufdeb.str(); // normalement 42 char
        // sortie sur fichier de la taille de buf2
        MPI_Offset offset = offset_LesCourbes1D[0]; // récup de l'offset
        int ierr = MPI_File_seek(*sort_MPI, offset, MPI_SEEK_SET); // positionnement
        if (ierr) // traitement erreur éventuelle
            {cout << "\n *** erreur de positionnement MPI_File_seek: code " << ierr << " , stop Herezh "
                  << "\n proc 0, ecriture buf0_LesCourbes1D "
                  << "\n  LesCourbes1D::Ecriture_base_info_MPI_IO(..." << endl;
             Sortie(1);
            };
        MPI_Status status; // pour le retour
        ierr = MPI_File_write(*sort_MPI,buf_inter.c_str(),42,MPI_CHAR,&status);
        if (ierr)
          {cout << "\n *** erreur d'acces a MPI_File_write: code " << ierr << " , stop Herezh "
                << "\n proc 0, ecriture entete buffer "
                << "\n  LesCourbes1D::Ecriture_base_info_MPI_IO(..." << endl;
           Sortie(1);
          };

      };

      // 2) ==== écriture des offsets c-a-d sortie sur fichier de buf2
      { MPI_Status status; // pour le retour
        int ierr = MPI_File_write(*sort_MPI,buf_inter.c_str(),taille_buf,MPI_CHAR,&status);
        if (ierr)
          {cout << "\n *** erreur d'acces a MPI_File_write: code " << ierr << " , stop Herezh "
                << "\n proc 0, ecriture des offsets "
                << "\n  LesCourbes1D::Ecriture_base_info_MPI_IO(..." << endl;
           Sortie(1);
          };
      };


      // 3) ==== écriture du  buffer propre
      // dans le cas cas != 1 le buffer est vide
      if (buffer_ioBI_MPI(1).size() > 0)
      { MPI_Status status; // pour le retour
        int ierr = MPI_File_write(*sort_MPI,buffer_ioBI_MPI(1).c_str()
                                     ,buffer_ioBI_MPI(1).size(),MPI_CHAR,&status);
        if (ierr)
            {cout << "\n *** erreur d'acces a MPI_File_write: code " << ierr << " , stop Herezh "
                  << "\n proc 0, ecriture premier buffer propre "
                  << "\n  LesCourbes1D::Ecriture_base_info_MPI_IO(..." << endl;
             Sortie(1);
         };
      };
     };
  
  };
  
#endif
     
// sortie du schemaXML: en fonction de enu  
void LesCourbes1D::SchemaXML_lesCourbes1D(ostream& sort,const Enum_IO_XML enu) 
 { switch (enu)
    {case XML_TYPE_GLOBAUX: // cas du premier passage
 	  {// cas des classes de base, on commence par créé la liste globale des courbes possible
       // on récupère la liste des identificateurs de courbes actuellement disponible
       list <EnumCourbe1D> list_enuCourbe1D = Courbe1D::Liste_courbe_disponible();
       // maintenant on définit un exemplaire de chaque courbe possible
       list <EnumCourbe1D>::iterator it,itfin=list_enuCourbe1D.end();
       int cot=1;string nom_courbe; Courbe1D * courbe=NULL;
       for (it=list_enuCourbe1D.begin();it!=itfin;it++,cot++)
         { nom_courbe = "exemple_courbe";
           nom_courbe += ChangeEntierSTring(cot);
           courbe = Courbe1D::New_Courbe1D(nom_courbe,*it);
           listeDeCourbe1D[nom_courbe]=courbe;
          };           
       break;		
      }
     case XML_IO_POINT_INFO:
      {// cas de def de  LesMaillages
   /*    sort  << "\n<xsd:element name=\"LesMaillages\" >"
  	          << "\n  <xsd:complexType>"
  	          << "\n    <xsd:element  name=\"****les_maillages:_nombre=\" type=\"xs:positiveInteger\" "
  	          << "\n              use=\"required\" minOccurs=\"1\" maxOccurs=\"unbounded\" />"
  	          << "\n    <xsd:element  name=\"defMaillages\" type=\"Maillage\" use=\"required\" />"
  	          << "\n  </xsd:complexType>"
              << "\n</xsd:element>";*/
     // def de Maillage
//     SchemaXML_Maillage(sort,niveau);
         break;		
      }
    };
  // on balaie l'ensemble des courbes
  map < string, Courbe1D * , std::less <string> >::iterator itc,itcfin =  listeDeCourbe1D.end();     
  for (itc= listeDeCourbe1D.begin();itc!=itcfin;itc++)
       (*itc).second->SchemaXML_Courbes1D(sort,enu);
    
 };


